# -*- coding: utf-8 -*-

from zope.interface import implements
from plone.memoize.instance import memoize
from sd.contents.interfaces import IBatchProvider
from sd.rendering.interfaces import IBatchedContentProvider


class BatchRendererMixin(object):
    """A batch mixin to render parts of content listing.
    """
    implements(IBatchedContentProvider)

    @property
    def batch_size(self):
        provider = IBatchProvider(self.context, None)
        return provider and provider.batch_size or 15

    @property
    def batch_name(self):
        return "batch_%s" % self.UID()

    @property
    def has_next_page(self):
        current_size = self.batch_size * (self.page + 1)
        return len(self.query_content()) > current_size

    def query_content(self):
        raise NotImplementedError("This method should return a list containing"
                                  " the content")

    def content(self):
        if not self.batch_size:
            return self.query_content()
        start = self.batch_size * self.page
        end = start + self.batch_size
        return self.query_content()[start:end]

    def get_page(self):
        return (getattr(self, '_page', None) or
                int(self.request.get(self.batch_name, 0)))

    def set_page(self, value):
        self._page = value
            
    page = property(get_page, set_page)


## Folderish
class FolderishContentProvider(BatchRendererMixin):
    u"""Basic content fetcher
    """
    @memoize
    def query_content(self):
        contentFilter = getattr(self, '_filtering', None)
        if contentFilter is None:
            return self.context.getFolderContents(full_objects = False)
        iface = {'object_provides': contentFilter}
        return self.context.getFolderContents(contentFilter=iface,
                                              full_objects = False)


## Topic
class TopicContentProvider(BatchRendererMixin):
    u"""Basic content fetcher
    """        
    @memoize
    def query_content(self):
        contentFilter = getattr(self, '_filtering', None)
        if contentFilter is None:
            return self.context.queryCatalog(full_objects = False)
        iface = {'object_provides': contentFilter}
        return self.context.queryCatalog(full_objects = False, **iface)


class TopicCustomRenderer(TopicContentProvider):
    u"""Custom view
    """    
    @property
    def fields(self):
        return self.context.getCustomViewFields()

    @property
    def metadatas(self):
        return self.context.listMetaDataFields(False)


class TopicChapterRenderer(object):
    u"""Full chapter rendering
    """
    @memoize
    def paragraphs(self):
        contentFilter = getattr(self, '_filtering', None)
        if contentFilter is None:
            return self.context.queryCatalog(full_objects = True)
        iface = {'object_provides': contentFilter}
        return self.context.queryCatalog(full_objects=True, **iface)
