# -*- coding: utf-8 -*-

from zope.component.zcml import adapter
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from sd.rendering.interfaces import IChapterRenderer, IParagraphRenderer
from Products.Five.metaclass import makeClass
from sd.rendering.base import ChapterStructuredRenderer, ParagraphStructuredRenderer

from zope.publisher.interfaces.browser import IBrowserRequest


def chapterDirective(_context, target, renderer=None, template=None,
                     name=u"default", filtering = None, description=None):
    u"""Register a chapter renderer.
    """    
    cname = "Chapter renderer %s" % name
    attrs = {}
        
    if template is not None:
        render = ViewPageTemplateFile(template)
        attrs['render'] = ViewPageTemplateFile(template)

    if renderer is None:
        attrs['__doc__'] = description or u"default renderer"
        baseclasses = (ChapterStructuredRenderer,)
    else:
        attrs['__doc__'] = description or renderer.__doc__
        baseclasses = (renderer, ChapterStructuredRenderer,)

    klass = makeClass(cname, baseclasses, attrs)
    if filtering:
        klass._filtering = filtering

    adapter(_context, (klass,), provides = IChapterRenderer,
            for_ = (target,IBrowserRequest), name = name, trusted=True)
    

def paragraphDirective(_context, target, renderer=None, template=None,
                       name=u"default", filtering = None, description=None):
    u"""Register a paragraph renderer.
    """
    cname = "Paragraph renderer %s" % name
    attrs = {}
        
    if template is not None:
        render = ViewPageTemplateFile(template)
        attrs['render'] = ViewPageTemplateFile(template)

    if renderer is None:
        attrs['__doc__'] = description or u"default renderer"
        baseclasses = (ParagraphStructuredRenderer,)
    else:
        attrs['__doc__'] = description or renderer.__doc__
        baseclasses = (renderer, ParagraphStructuredRenderer,)
        
    klass = makeClass(cname, baseclasses, attrs)
    if filtering:
        klass._filtering = filtering
    
    adapter(_context, (klass,), provides = IParagraphRenderer,
            for_ = (target,IBrowserRequest), name = name, trusted=True)
