# -*- coding: utf-8 -*-

from Acquisition import Explicit
from zope.interface import implements
from plone.memoize.instance import memoize
from Products.Five.browser import BrowserView
from sd.contents.interfaces import IUndirectLayoutProvider
from sd.contents.interfaces import IDynamicStructuredChapter
from sd.contents.interfaces import IDynamicStructuredParagraph
from interfaces import IStructuredChapterView
from interfaces import IStructuredContentProvider
from interfaces import IBaseStructuredRenderer


class BaseStructuredContentProvider(Explicit):
    u"""The base implementation of the structured content provider"""
    
    implements(IStructuredContentProvider)
    
    def __init__(self, context, request, view):
        self.context = context
        self.request = request
        self.__parent__ = view

    def update(self):
        pass

    def render(self):
        raise NotImplementedError(u"You must implement 'render' as a method "
                                  u"or page template file attribute")



class BaseStructuredRenderer(BrowserView):
    u"""The base implementation of the structured renderer"""

    implements(IBaseStructuredRenderer, IUndirectLayoutProvider)

    _filtering = None

    def getId(self):
        return self.context.getId()

    def UID(self):
        return self.context.UID()

    def Title(self):
        return self.context.Title()
    
    def Description(self):
        return self.context.Description()

    def absolute_url(self):
        return self.context.absolute_url()

    def widget(self, *args, **kwargs):
        return self.context.widget(*args, **kwargs)

    def render(self):
        raise NotImplementedError(u"You must implement 'render' as a method "
                                  u"or page template file attribute")


class ChapterStructuredRenderer(BaseStructuredRenderer):
    
    implements(IStructuredChapterView)

    @property
    def show_title(self):
        return IDynamicStructuredChapter(self.context).show_title

    @property
    def show_description(self):
        return IDynamicStructuredChapter(self.context).show_description

    @memoize
    def paragraphs(self):
        iface  = {"object_provides":
                  'sd.contents.interfaces.IStructuredParagraph' }
        return self.context.getFolderContents(contentFilter = iface,
                                              full_objects = True)


class ParagraphStructuredRenderer(BaseStructuredRenderer):

    @property
    def show_title(self):
        return IDynamicStructuredParagraph(self.context).show_title

    @property
    def show_description(self):
        return IDynamicStructuredParagraph(self.context).show_description
